/* <link rel="stylesheet" type="text/css" href="example1.css"/> */

div > div
{
	background: #fd9;
	border: 1px solid #db7;
	margin: 1em;
	padding: 0.67em;
}

div > div h3
{
	margin: 0;
	padding: 0;
}

div > div span.rdf-type
{
	float: right;
	text-align: right;
	width: 40px;
	position: relative;
	top: -1.67em;
}

div > div dl dt
{
	margin: 0.5em 0 0;
	padding: 0;
	font-weight: bold;
	font-style: italic;
	font-size: smaller;
}

dd.seeAlso a, a[rel="seeAlso"]
{
	background: transparent url("data:image/gif;base64,iVBORw0KGgoAAAANSUhEUgAAABAAAAAQCAYAAAAf8/9hAAAABGdBTUEAAK/INwWK6QAAABl0RVh0U29mdHdhcmUAQWRvYmUgSW1hZ2VSZWFkeXHJZTwAAAEsSURBVDjLY/j//z8DJZhhmBuQvdj8a+p8w//xc3U5yTIgeb7h18bNUf/DZ2j8958qz0nQgPyltv9zF5v/TV9o/Ddxrv7fmvWh/1ednvi/ZLX/f9d+8b+23YI8eA0AOvn/hnPT/q89OwWsccXpCf8n7Cn5v/B41/+MpW7/TdvZ/+o2M/LjNADoZLDmvl35/zt3ZP9v3Zb2v2Fz4v+mren/Zxxq/h+zwOa/aj3DH5wGRM/W/L/y1IT/S0/0/l94rOv/vKMdQEOy/k8/1PQ/banHf8VahlfSlQwGOA0InK74z3Oi9D/nftF/1t38/+LmW/2fdrDhf9Jit//ytQzPJSsZtEiKBe1mxq/xC53/y1czPAFqVic5GoFO/ipXzfxftJJBkeyUKFzOwDm48wIAh5XH+g7drOwAAAAASUVORK5CYII=") no-repeat scroll center left;
	color: #393;
	padding: 0 0 0 16px;
	text-decoration: none;
	font-size: 80%;
}
dd.seeAlso a[rel="seeAlso"]
{
	font-size: 100%;
}